import argparse
import openpyxl as px
from openpyxl.styles import Font, PatternFill, Border, Alignment
from openpyxl.utils import get_column_letter
from datetime import datetime

# コマンドライン引数のパース
print("Parsing command line arguments...")
parser = argparse.ArgumentParser()
parser.add_argument("input_file", help="The name of the input Excel file.")
args = parser.parse_args()

# Excelファイルの読み込み
print("Loading Excel file...")
wb = px.load_workbook(args.input_file)

# カラム行番号
column_row_idx = 3

# 新しいExcelファイル名の作成
output_file = f'{args.input_file}_{datetime.now().strftime("%y%m%d%H%M%S")}.xlsx'
print(f"Output file will be: {output_file}")

# 色の設定
fill_color_1 = PatternFill(fill_type="solid", fgColor="C6E0B4")  # Mint green
fill_color_2 = PatternFill(fill_type="solid", fgColor="9BC2E6")  # Light blue

# Save original column widths
col_widths = {}

# シートごとに処理
for i in range(len(wb.sheetnames)):
    sheet_name_n = f'影響{i}' if i > 0 else '影響'  # "影響"シートを読み込む
    if i != len(wb.sheetnames)-1:
        sheet_name_n_plus_1 = f'影響{i+1}'  # 次の"影響"シートを読み込む
    print(f"Processing sheet: {sheet_name_n}")
    ws_n = wb[sheet_name_n]
    if i != len(wb.sheetnames)-1:
        ws_n_plus_1 = wb[sheet_name_n_plus_1]


    # '間接影響先有'列の挿入
    ws_n.insert_cols(5)
    ws_n.cell(row=column_row_idx, column=5).value = '間接影響先有'

    # 各行に対して処理
    fill_color = fill_color_1
    prev_element = ws_n.cell(row=column_row_idx+1, column=1).value  # 最初の要素を取得
    for j in range(column_row_idx+1, ws_n.max_row + 1):  # 4行目から処理を開始
        element = ws_n.cell(row=j, column=1).value  # 'サブシステム'列の値を取得

        # 要素が前の行と異なる場合、色を切り替える
        if element != prev_element:
            fill_color = fill_color_2 if fill_color == fill_color_1 else fill_color_1
            prev_element = element

        # A列からD列までのセルの背景色を設定
        for k in range(1, 5):
            ws_n.cell(row=j, column=k).fill = fill_color

        # 'Used By'列の値を取得
        if i != len(wb.sheetnames)-1:
            used_by_element = ws_n.cell(row=j, column=3).value

        # 次のレベルのシートに要素が存在するか確認
        if i != len(wb.sheetnames)-1:
            if used_by_element in [cell.value for cell in ws_n_plus_1['A']]:
                # 要素が初めて現れる位置を取得
                target_row = [cell.value for cell in ws_n_plus_1['A']].index(used_by_element) + 1

                # ハイパーリンクの作成
                print(f"Creating hyperlink for element: {used_by_element}")
                ws_n.cell(row=j, column=3+1).hyperlink = f"#{sheet_name_n_plus_1}!B{target_row}"  # 'Used By'列にハイパーリンクを設定 #あとでInsertする。hyperlinkだけInsertで右にずれてくれないのでcolumnの設定を+1している。
                ws_n.cell(row=j, column=3).value = used_by_element  # リンクテキストを元の値に設定
                ws_n.cell(row=j, column=5).value = "○" #次のシートへのリンクがある場合はあとでユーザーがフィルターできるように表記する。

                # フォントを太字と斜体に設定
                ws_n.cell(row=j, column=3).font = Font(color="0000EE", bold=True, italic=True, underline="single")  # 0000EEはハイパーリンクの色

    # 'No'列の挿入
    ws_n.insert_cols(1)
    ws_n.cell(row=column_row_idx, column=1).value = 'No.'



    # 'No'列の各行に対して処理
    for j in range(column_row_idx+1, ws_n.max_row + 1):  # 4行目から処理を開始
        # 行番号の設定と背景色の設定
        ws_n.cell(row=j, column=1).value = j - column_row_idx
        ws_n.cell(row=j, column=1).fill = fill_color
        bj_fill = ws_n.cell(row=j, column=2).fill
        ws_n.cell(row=j, column=1).fill = PatternFill(fill_type=bj_fill.fill_type, start_color=bj_fill.start_color, end_color=bj_fill.end_color)
    
    
    # A3セル（'No'列のカラム名）の書式をB3セルと同じにする
    b3_font = ws_n.cell(row=column_row_idx, column=2).font
    b3_fill = ws_n.cell(row=column_row_idx, column=2).fill
    b3_border = ws_n.cell(row=column_row_idx, column=2).border
    b3_alignment = ws_n.cell(row=column_row_idx, column=2).alignment

    ws_n.cell(row=column_row_idx, column=1).font = Font(name=b3_font.name, size=b3_font.size, bold=b3_font.bold, italic=b3_font.italic, 
                                            vertAlign=b3_font.vertAlign, underline=b3_font.underline, strike=b3_font.strike,
                                            color=b3_font.color)
    ws_n.cell(row=column_row_idx, column=1).fill = PatternFill(fill_type=b3_fill.fill_type, start_color=b3_fill.start_color, end_color=b3_fill.end_color)
    ws_n.cell(row=column_row_idx, column=1).border = Border(left=b3_border.left, right=b3_border.right, top=b3_border.top, bottom=b3_border.bottom, 
                                               diagonal=b3_border.diagonal, diagonal_direction=b3_border.diagonal_direction, 
                                               outline=b3_border.outline, vertical=b3_border.vertical, horizontal=b3_border.horizontal)
    ws_n.cell(row=column_row_idx, column=1).alignment = Alignment(horizontal=b3_alignment.horizontal, vertical=b3_alignment.vertical,
                                                     text_rotation=b3_alignment.text_rotation, wrap_text=b3_alignment.wrap_text,
                                                     shrink_to_fit=b3_alignment.shrink_to_fit, indent=b3_alignment.indent)

    # '間接影響先有'列の各行に対して処理
    for j in range(column_row_idx+1, ws_n.max_row + 1):  # 4行目から処理を開始
        # 行番号の設定と背景色の設定
        ws_n.cell(row=j, column=6).fill = fill_color
        bj_fill = ws_n.cell(row=j, column=2).fill
        ws_n.cell(row=j, column=6).fill = PatternFill(fill_type=bj_fill.fill_type, start_color=bj_fill.start_color, end_color=bj_fill.end_color)



    # F3セル（'間接影響先有'列のカラム名）の書式をB3セルと同じにする
    ws_n.cell(row=column_row_idx, column=6).font = Font(name=b3_font.name, size=b3_font.size, bold=b3_font.bold, italic=b3_font.italic, 
                                            vertAlign=b3_font.vertAlign, underline=b3_font.underline, strike=b3_font.strike,
                                            color=b3_font.color)
    ws_n.cell(row=column_row_idx, column=6).fill = PatternFill(fill_type=b3_fill.fill_type, start_color=b3_fill.start_color, end_color=b3_fill.end_color)
    ws_n.cell(row=column_row_idx, column=6).border = Border(left=b3_border.left, right=b3_border.right, top=b3_border.top, bottom=b3_border.bottom, 
                                               diagonal=b3_border.diagonal, diagonal_direction=b3_border.diagonal_direction, 
                                               outline=b3_border.outline, vertical=b3_border.vertical, horizontal=b3_border.horizontal)
    ws_n.cell(row=column_row_idx, column=6).alignment = Alignment(horizontal=b3_alignment.horizontal, vertical=b3_alignment.vertical,
                                                     text_rotation=b3_alignment.text_rotation, wrap_text=b3_alignment.wrap_text,
                                                     shrink_to_fit=b3_alignment.shrink_to_fit, indent=b3_alignment.indent)



    # 列の幅を調整
    ws_n.column_dimensions['A'].width = 5
    ws_n.column_dimensions['B'].width = 50
    ws_n.column_dimensions['C'].width = 35
    ws_n.column_dimensions['D'].width = ws_n.column_dimensions['B'].width
    ws_n.column_dimensions['E'].width = 10
    ws_n.column_dimensions['F'].width = 15
    
    # フィルターを設定
    ws_n.auto_filter.ref = 'A'+str(column_row_idx)+':F'+str(ws_n.max_row)


# Excelファイルに保存
print("Saving Excel file...")
wb.save(filename=output_file)
print("Done.")